@@ -1,15 +1,17 @@
 package Spreadsheet::XLSX;
 
+use 5.008008;
 use strict;
 use warnings;
 
 our @ISA = qw();
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 use Archive::Zip;
-
+use Spreadsheet::XLSX::Fmt2007;
 use Data::Dumper;
+use Spreadsheet::ParseExcel;
 
 ################################################################################
 
@@ -27,8 +29,11 @@ sub new {
 
 	if ($member_shared_strings) {
 	
-		foreach my $t ($member_shared_strings -> contents =~ /\>([^\<]*)\<\/t/gsm) {
-			
+		my $mstr = $member_shared_strings->contents; 
+		$mstr =~ s/<t\/>/<t><\/t>/gsm;  # this handles an empty t tag in the xml <t/>
+
+		#foreach my $t ($member_shared_strings -> contents =~ /t\>([^\<]*)\<\/t/gsm) {
+		foreach my $t ($mstr =~ /<t.*?>(.*?)<\/t/gsm) {
 			$t = $converter -> convert ($t) if $converter;
 			
 			push @shared_strings, $t;
@@ -36,9 +41,47 @@ sub new {
 		}
 	
 	}
+
+        my $member_styles = $self -> {zip} -> memberNamed ('xl/styles.xml');
+
+        my @styles = ();
+
+	my %style_info = ();
+
+        if ($member_styles) {
+
+                foreach my $t ($member_styles -> contents =~ /xf\ numFmtId="([^"]*)"(?!.*\/cellStyleXfs)/gsm) { #"
+                       # $t = $converter -> convert ($t) if $converter;
+                        push @styles, $t;
+
+                }
+		my $default = $1;
+
+		foreach my $t1 (@styles){
+			$member_styles -> contents =~ /numFmtId="$t1" formatCode="([^"]*)/;
+			my $formatCode=$1;
+			if ($formatCode eq $default || not($formatCode)){
+				if ($t1 == 9 || $t1==10){ $formatCode="0.00000%";}
+				elsif ($t1 == 14){ $formatCode="m-d-yy";}
+				else {
+					$formatCode="";
+				}
+			}
+			$style_info{$t1} = $formatCode;
+			$default=$1;
+		}
+
+        }
+
 		
 	my $member_workbook = $self -> {zip} -> memberNamed ('xl/workbook.xml') or die ("xl/workbook.xml not found in this zip\n");
-			
+	my $oBook = Spreadsheet::ParseExcel::Workbook->new;
+	$oBook->{SheetCount} = 0;
+	$oBook->{FmtClass} = Spreadsheet::XLSX::Fmt2007->new;
+	$oBook->{Flg1904}=0;
+	if ($member_workbook->contents =~ /date1904="1"/){
+		$oBook->{Flg1904}=1;
+	}
 	my @Worksheet = ();
 	
 	foreach ($member_workbook -> contents =~ /\<(.*?)\/?\>/g) {
@@ -60,7 +103,7 @@ sub new {
 		
 		foreach ($other =~ /(\S+=".*?")/gsm) {
 
-			my ($k, $v) = split /=?"/;
+			my ($k, $v) = split /=?"/; #"
 	
 			if ($k eq 'name') {
 				$sheet -> {Name} = $v;
@@ -71,8 +114,10 @@ sub new {
 			};
 					
 		}
-		
-		push @Worksheet, $sheet;
+		my $wsheet = Spreadsheet::ParseExcel::Worksheet->new(%$sheet);
+		push @Worksheet, $wsheet;
+		$oBook->{Worksheet}[$oBook->{SheetCount}] = $wsheet;
+		$oBook->{SheetCount}+=1;
 				
 	}
 
@@ -88,23 +133,26 @@ sub new {
 		
 		my $flag = 0;
 		my $s    = 0;
-		
+		my $s2   = 0;
+		my $sty  = 0;
 		foreach ($member_sheet -> contents =~ /(\<.*?\/?\>|.*?(?=\<))/g) {
-		
 			if (/^\<c r=\"([A-Z])([A-Z]?)(\d+)\"/) {
 				
 				$col = ord ($1) - 65;
 				
 				if ($2) {
-         				$col++;
+                			$col++;
 					$col *= 26;
 					$col += (ord ($2) - 65);
 				}
 				
 				$row = $3 - 1;
 				
-				$s = /t=\"s\"/ ? 1 : 0;
-				
+				$s  = /t=\"s\"/ ? 1 : 0;
+				$s2 = /t=\"str\"/ ? 1 : 0;
+				/s="([^"]*)"/; #"
+
+				$sty = $1>0 ? $1 : 0 ;
 			}
 			elsif (/^<v/) {
 				$flag = 1;
@@ -113,21 +161,38 @@ sub new {
 				$flag = 0;
 			}
 			elsif (length ($_) && $flag) {
-			
 				my $v = $s ? $shared_strings [$_] : $_;
-			
+				if ($v eq "</c>"){$v="";}
+				my $type = "Text";
+				my $thisstyle = "";
+				if (not($s) && not($s2)){
+					$type="Numeric";
+					$thisstyle = $style_info{$styles[$sty]};
+					if ($thisstyle =~ /(?<!Re)d|m|y/){
+						$type="Date";
+					}
+				}	
 				$sheet -> {MaxRow} = $row if $sheet -> {MaxRow} < $row;
 				$sheet -> {MaxCol} = $col if $sheet -> {MaxCol} < $col;
 				$sheet -> {MinRow} = $row if $sheet -> {MinRow} > $row;
 				$sheet -> {MinCol} = $col if $sheet -> {MinCol} > $col;
-				
-				$sheet -> {Cells} [$row] [$col] = {
+				if ($v =~ /(.*)E\-(.*)/gsm && $type eq "Numeric"){
+					$v=$1/(10**$2);  # this handles scientific notation for very small numbers
+				}
+				my $cell =Spreadsheet::ParseExcel::Cell->new(
 
 					Val    => $v,
-					_Value => $v,
+					Format => $thisstyle,
+					Type => $type
 					
-				};
-			
+				);
+
+				$cell->{_Value} = $oBook->{FmtClass}->ValFmt($cell, $oBook);
+				if ($type eq "Date" && $v<1){  #then this is Excel time field
+					$cell->{Type}="Text";
+					$cell->{Val}=$cell->{_Value};
+				}
+				$sheet -> {Cells} [$row] [$col] = $cell;
 			}
 					
 		}
@@ -136,10 +201,11 @@ sub new {
 		$sheet -> {MinCol} = 0 if $sheet -> {MinCol} > $sheet -> {MaxCol};
 
 	}
-	
+foreach my $stys (keys %style_info){
+}
 	bless ($self, $class);
 
-	return $self;
+	return $oBook;
 
 }
 
@@ -189,7 +255,9 @@ Spreadsheet::XLSX - Perl extension for reading MS Excel 2007 files;
 =head1 DESCRIPTION
 
 This module is a (quick and dirty) emulation of Spreadsheet::ParseExcel for 
-Excel 2007 (.xlsx) file format.
+Excel 2007 (.xlsx) file format.  It supports styles and many of Excel's quirks, 
+but not all.  It populates the classes from Spreadsheet::ParseExcel for interoperability; 
+including Workbook, Worksheet, and Cell.
 
 =head1 SEE ALSO
 
@@ -246,7 +314,8 @@ Patches by:
 
 	Steve Simms
 	Joerg Meltzer
-	Loreyna Yeung
+	Loreyna Yeung	
+	Rob Polocz
 
 =head1 COPYRIGHT AND LICENSE
 
